//-----------------------------------------------------------------------------
// Macro: BOX
//
// Form:  BOX  [SINGLE | DOUBLE]
//
//   SINGLE - optional parameter, sets the box style to a single line box.
//   This is the default style. Once set the box style is remembered as
//   long a you do not exit SPF/SE.
//    
//   DOUBLE - optional parameter, sets the box style to a double line box.
//   Once set the box style is remembered as long a you do not exit SPF/SE.
//
// Function:
//
//   This macro draws a box. The current cursor position marks the top left
//   left corner of the box. After entering the BOX command, move the cursor to
//   the desired position for the lower right corner of the box, then enter the
//   BOX command a second time. (Note: we recommend that you map BOX to a PF
//   key. Press it, move the cursor, and press it a second time.)
// 
//   The BOX macro automatically senses the font and sets appropriate character 
//   codes for box draw characters.
//
// NOTE: Most example macros provide trivial functionality. Their primary
// purpose is to show a variety of coding techniques.
//-----------------------------------------------------------------------------
void box(int argc, char *argv[]) {
 
  char *font_face;
  char *cmd;
  char *raw_response;  // receiver for raw query response
  char *cur_line;
  char *buf;
  char *msg;
   
  // BOX draw characters
  char *rule_h, *rule_v;
  char *corner_tl, *corner_tr, *corner_bl, *corner_br;
   
  int   col_left;
  int   col_right;
  int   cur_col;
  int   i;
   
  BOOL  box_pending;
  BOOL  error = FALSE;         
  BOOL  safe  = TRUE;
  BOOL  single;
   
  buf = SPFservice("vget","box.pending");
  if (buf == "")
    SPFservice("vput","box.pending","FALSE"); // not initialized, set default value
     
  buf = SPFservice("vget","box.single");   
  if (buf == "")
    SPFservice("vput","box.single","TRUE");   // not initialized, set default value
   
  // Below are displayed the line draw characters from font "Terminal".
  // See menu OPTIONS, item DISPLAY, button EDIT/BROWSE FONT
   
  // xB3..xBF:                    
   
  // xC0..xCF:                
   
  // xD0..xDA:            
   
  if (argc == 2) {
    //------------------
    // process parameter
    //------------------
    switch (argv[1]) {
      case "single": SPFservice("vput","box.single","TRUE");  break;
      case "double": SPFservice("vput","box.single","FALSE"); break;
      default:    
        error = TRUE;
        msg = StrCompose("BOX: '",argv[1],"' is not a valid parameter");
        Msg(msg);
        break;
    } // switch
  }
  if (!error) {
    cur_col  = SPFservice("query","col_number");      // get current data column number
    cur_line = SPFservice("query","line_number");     // get current data line number
    if (cur_col > 0 && cur_line > 0) {                // in edit client area?
      box_pending = SPFservice("vget","box.pending");
      if (!box_pending) {
        //-------------------------------------------------------
        // 1st invokation, cursor pos is upper left corner of box
        //-------------------------------------------------------
        SPFservice("vput","box.pending","TRUE");      // set "box pending"
        SPFservice("vput","box.top.line",cur_line);   // save top line of box
        SPFservice("vput","box.col.left",cur_col);    // save left col of box
        SPFservice("cmd","reset label");              // reset old labels
        }
      else {
        //----------------------------------------------------------------
        // 2nd invokation, current cursor pos is lower right corner of box
        //----------------------------------------------------------------
        SPFservice("vput","box.pending","FALSE");     // reset
        single = SPFservice("vget","box.single");     // get box line style
        font_face = SPFservice("query","font_face");  // get current font
        if (font_face == "Terminal") {
          //---------------------------------------
          // set box draw chars ASCII character set
          //---------------------------------------
          if (single) {
            rule_h    = ""; // horizontal rule char
            rule_v    = ""; // vertical rule char
            corner_tr = ""; // top right corner char
            corner_tl = ""; // top left corner char
            corner_br = ""; // bottom right corner char
            corner_bl = ""; // bottom left corner char
            }
          else {
            rule_h    = ""; // horizontal rule char
            rule_v    = ""; // vertical rule char
            corner_tr = ""; // top right corner char
            corner_tl = ""; // top left corner char
            corner_br = ""; // bottom right corner char
            corner_bl = ""; // bottom left corner char
          }
          }
        else {
          //--------------------------------------
          // set box draw chars ANSI character set
          //--------------------------------------
          if (single) {
            rule_h    = "-"; // horizontal rule char
            rule_v    = "|"; // vertical rule char
            corner_tl = "+"; // top left corner char
            corner_tr = "+"; // top right corner char
            corner_bl = "+"; // bottom left corner char
            corner_br = "+"; // bottom right corner char
            }
          else {
            rule_h    = "="; // horizontal rule char
            rule_v    = "|"; // vertical rule char
            corner_tl = "+"; // top left corner char
            corner_tr = "+"; // top right corner char
            corner_bl = "+"; // bottom left corner char
            corner_br = "+"; // bottom right corner char
          }
        }
        col_left = SPFservice("vget","box.col.left"); // get value from 1st invokation
        col_right = cur_col;                  // get current data column
        if (col_right > col_left) {
          SPFservice("cmd",":.bxbot");        // label bottom line of box
          cur_line = SPFservice("vget","box.top.line"); // get saved top line number
          // position to top line of box and label it
          cmd = StrCompose("locate ",cur_line,";:.bxtop");
          SPFservice("cmd",cmd);

          // check box outline path for non-blanks

          // check top of box
          cmd = StrCompose("find quiet p'^' .bxtop .bxtop ",
                  col_left," ",col_right);
          SPFservice("cmd",cmd);
          i = SPFservice("query","find_string_count");
          if (i) safe = FALSE;

          // check bot of box
          cmd = StrCompose("find quiet p'^' .bxbot .bxbot ",
                  col_left," ",col_right);
          SPFservice("cmd",cmd);
          i = SPFservice("query","find_string_count");
          if (i) safe = FALSE;

          // check left edge of box
          cmd = StrCompose("find quiet p'^' .bxtop .bxbot ",
                  col_left," ",col_left);
          SPFservice("cmd",cmd);
          i = SPFservice("query","find_string_count");
          if (i) safe = FALSE;

          cmd = StrCompose("find quiet p'^' .bxtop .bxbot ",
                  col_right," ",col_right);
          SPFservice("cmd",cmd);
          i = SPFservice("query","find_string_count");
          if (i) safe = FALSE;

          if (safe) {

            //----------------
            // draw top of box
            //----------------
            cmd = StrCompose("change quiet all p'=' ",rule_h," .bxtop .bxtop ",
                    col_left," ",col_right);
            SPFservice("cmd",cmd);

            //-------------------
            // draw bottom of box
            //-------------------
            cmd = StrCompose("change quiet all p'=' ",rule_h," .bxbot .bxbot ",
                    col_left," ",col_right);
            SPFservice("cmd",cmd);

            //----------------------
            // draw left edge of box
            //----------------------
            cmd = StrCompose("change quiet all p'=' ",rule_v," .bxtop .bxbot ",
                    col_left," ",col_left);
            SPFservice("cmd",cmd);

            //-----------------------
            // draw right edge of box
            //-----------------------
            cmd = StrCompose("change quiet all p'=' ",rule_v," .bxtop .bxbot ",
                    col_right," ",col_right);
            SPFservice("cmd",cmd);

            //---------------------
            // draw top left corner
            //---------------------
            cmd = StrCompose("change quiet all p'=' ",corner_tl," .bxtop .bxtop ",
                    col_left," ",col_left);
            SPFservice("cmd",cmd);

            //----------------------
            // draw top right corner
            //----------------------
            cmd = StrCompose("change quiet all p'=' ",corner_tr," .bxtop .bxtop ",
                    col_right," ",col_right);
            SPFservice("cmd",cmd);

            //------------------------
            // draw bottom left corner
            //------------------------
            cmd = StrCompose("change quiet all p'=' ",corner_bl," .bxbot .bxbot ",
                    col_left," ",col_left);
            SPFservice("cmd",cmd);

            //-------------------------
            // draw bottom right corner
            //-------------------------
            cmd = StrCompose("change quiet all p'=' ",corner_br," .bxbot .bxbot ",
                    col_right," ",col_right);
            SPFservice("cmd",cmd);

            // reposition to top of box, then reset labels and highlights
            SPFservice("cmd","locate .bxtop;reset;reset label");

            // reset saved values
            SPFservice("vput","box.top.line","");
            SPFservice("vput","box.col.left","");

            }
          else {
            error = TRUE;
            Msg("BOX: some characters in the box outline path are non-blank, can not proceed.");
            SPFservice("cmd","reset;reset label");
          }
          }
        else {
          error = TRUE;
          Msg("BOX: second cursor position ABOVE or LEFT of first, can not proceed.");
          SPFservice("cmd","reset;reset label");
        }
      }
      }
    else {
      error = TRUE;
      Msg("BOX: cursor is not in data area.");
    }
  }
  if (error) {
    // error occured, reset possible pending state info
    SPFservice("vput","box.col.left","0");
    SPFservice("vput","box.pending","FALSE");
    SPFservice("vput","box.top.line","0");
  }
 
} // box
