//------------------------------------------------------------------------
//
// Macro: PARSEWORDS
//
// Form:  PARSEWORDS
//
// Function:
//
//   Parse a file into individual words delimited on any non-alpha character.
//   Blank or empty lines are not preserved.
//
//------------------------------------------------------------------------
void ParseWords(void) {
 
  char *command;
  char *text;
  char *word;
  char *testchar;
  char *words[];
  int   word_count = 0;
  int   tasks;
  int   cur_col;
  int   textlen;
  int   wordlen;
  int   line_count;
  int   cur_line = 1;
  int   word_start_col;
  int   i;
  BOOL  allblank;
  BOOL  blanks_done;
  BOOL  word_done;
  BOOL  allblank;
   
  tasks = SPFservice("query","task_count");
  if (tasks > 0) {
    //-----------------------------------------------------------
    // parse each line of current edit file into individual words
    //-----------------------------------------------------------
    line_count = SPFservice("query","line_count");
    while (cur_line <= line_count) {
      command = StrCompose("locate ",cur_line);
      SPFservice("cmd",command);              // position to next line
      text = SPFservice("query","line_data"); // retrieve line data
      textlen = strlen(text);                 // get line length
      if (textlen > 0) {                      // length greater than 0?
        //-------------------------
        // parse line left to right
        //-------------------------
        cur_col = 1;
        while (cur_col <= textlen) {          // more chars to parse?
          //---------------------
          // strip leading blanks
          //---------------------
          blanks_done = FALSE; // reset
          while (!blanks_done && cur_col <= textlen) {
            testchar = StrGetSubstr(text,cur_col,1); // get next char
            if (testchar == " ") {                   // is char blank?
              ++cur_col;                             // step over leading blank
              }
            else {
              blanks_done = TRUE;                    // nonblanks ends loop
            }
          } // while
          //----------------
          // parse next word
          //----------------
          word_done = FALSE; // reset
          wordlen = 0;
          word_start_col = cur_col;
          while (!word_done && cur_col <= textlen) {
            testchar = StrGetSubstr(text,cur_col,1); // get next char
            if (StrIsAlpha(testchar)) {              // is char alpha?
              ++wordlen;                             // incr word length
              ++cur_col;                             // go to next col
              }
            else {
              word_done = TRUE;                      // word ends on non-alpha
            }
          } // while
          if (wordlen > 0) {
            //-----------
            // word found
            //-----------
            word = StrGetSubstr(text,word_start_col,wordlen); // get word
            words[word_count] = word;
            ++word_count;
            cur_col = word_start_col + wordlen; // resume
            }
          else {
            if (cur_col <= textlen) {
              //--------------------------------------
              // current char non-alpha, treat as word
              //--------------------------------------
              word = StrGetSubstr(text,cur_col,1); // get non-alpha char
              if (word != " ") {                   // char non-blank?
                if (word == ";") {
                  words[word_count] = "';'";       // special case, put ";" in single quotes
                  }
                else {
                  if (word == "\"") {
                    words[word_count] = "'\"'";       // special case, put ";" in single quotes
                    }
                  else {
                    words[word_count] = word;        // treat non-alpha char as word
                  }
                }
                ++word_count;                      // incr word count
                ++cur_col;
              }
            }
          }
        } // while
      }
      ++cur_line;
    } // while
    if (word_count > 0) {
      //-----------------------------
      // delete current file contents
      //-----------------------------
      SPFservice("cmd","ex all;delete x");
      //--------------------------------------
      // insert each individual word as a line
      //--------------------------------------
      i = 0;
      while (i < word_count) {
        SPFservice("cmd","locate .zlast");
        SPFservice("set","cursor_row","1"); // put cursor on top display line
        command = StrCompose("dataline ",words[i]);
        SPFservice("cmd",command);
        ++i;
      } // while
      SPFservice("cmd","top");
      Msg(StrCompose("Parsed ",word_count," words"));
    }
    }
  else {
    Msg("Not in edit, macro ignored.");
  }
 
} // ParseWords
