//------------------------------------------------------------------------------
// This is an example of how a user might initialize and launch a simple dialog.
// The sample dialog "SpfDialog.user1.txt" has a variety of field types. The dialog
// definition specifies the macro to get control when the dialog commits. If the
// dialog is cancelled, normal SPF/SE processing resumes.
//
// In all cases, the call to SPFservice "dialog" must be the last statement
// in the macro which launches a dialog.
//
// NOTE: The name of this macro does not have to match the dialog name.
//------------------------------------------------------------------------------
void User1(void) {

  char *text;
  char *choice;
  char *cur_item;
  char *lcmd;
  char *item_varname;
  char *DLG_RC;
  int   cur_item_number;
  int   item_count;
  int   number;
  BOOL  boolval;
  BOOL  any_error = FALSE;

  spfservice("vput","User1.TextIn","this is some input text"); // initialize text var
  spfservice("vput","User1.NumIn","99");                       // initialize number var
  spfservice("vput","User1.ChoiceIn","Choice2");               // initialize choice var
  spfservice("vput","User1.BoolIn","TRUE");                    // initialize boolean var
  spfservice("vput","User1.List.0","5");                       // set number of items in list
  // set list items
  spfservice("vput","User1.List.1","aaa (field1 content)   bbb (field2 content)");
  spfservice("vput","User1.List.2","ccc (field1 content)   ddd (field2 content)");
  spfservice("vput","User1.List.3","eee (field1 content)   fff (field2 content)");
  spfservice("vput","User1.List.4","ggg (field1 content)   hhh (field2 content)");
  spfservice("vput","User1.List.5","iii (field1 content)   jjj (field2 content)");
  DLG_RC = spfservice("dialog","User1");                       // launch dialog
  if (DLG_RC == "OK") {
    //---------------------------------------------------------------
    // This code gets control; when the dialog commits via 3270-ENTER
    //---------------------------------------------------------------
    text    = spfservice("vget","User1.TextIn");          // retrieve dialog variable
    choice  = spfservice("vget","User1.ChoiceIn");        // retrieve dialog variable
    number  = spfservice("vget","User1.NumIn");           // retrieve dialog variable
    boolval = spfservice("vget","User1.BoolIn");          // retrieve dialog variable
    msg(strcompose("Current field values:\n\nText=",text,"\nNumber=",number,"\nChoice=",choice,"\nBool=",boolval));
    item_count = spfservice("vget","User1.List.REPLY.0"); // retrieve number of items with verb
    if (item_count > 0) {
      lcmd = spfservice("vget","User1.POPUP.REPLY");      // retrieve line command
      strupr(lcmd);                                       // make line command upcase
      cur_item_number = 1;
      while (!any_error && cur_item_number <= item_count) {
        strcpy(item_varname,"User1.List.REPLY.");
        strcat(item_varname,cur_item_number);       // synthesize varname to retrieve item
        cur_item = spfservice("vget",item_varname); // retrieve list item
        switch (lcmd) {

          case "SELECT":
            // ... this is where you put your action code ...
            Msg(strcompose("Apply SELECT to:  ",cur_item));
            break;

          case "RENAME":
            // ... this is where you put your action code ...
            Msg(strcompose("Apply RENAME to:  ",cur_item));
            break;

          case "DELETE":
            // ... this is where you put your action code ...
            Msg(strcompose("Apply DELETE to:  ",cur_item));
            break;

          case "MOVETOTOP":
            // ... this is where you put your action code ...
            Msg(strcompose("Apply MOVETOTOP to:  ",cur_item));
            break;

          default:
            any_error = TRUE;
            Msg(strcompose("List item  ",cur_item_number,"  has unknown line command:  ",lcmd));
            break;

        } // switch
        ++cur_item_number;
      } // while
    }

    //-----------------------------------------------------------------------
    // The following optional call causes the macro to be executed again
    // producing the effect of "loop until cancel". If you do not have a need
    // for this, delete following "spfservice()" statement.
    //
    // NOTE: If used, this must be the last statement in your macro.
    //-----------------------------------------------------------------------
    spfservice("macro","user1"); // re-execute self after macro ends
  }

} // User1
