//--------------------------------------------------------------------------------------
// 1) Map any PF Key, CTRL+char key, or ALT+char key to "EditFileAtCursor"
// 2) File names may NOT have embedded blanks.
// 3) To Edit file in specified path,
//    put cursor on first letter of fully qualified filename (e.g. "C" of "C:\...")
//
//    C:\temp\abc.txt ...line-data... C:\temp\def.txt ...line-data... C:\temp\ghi.txt
//
// 4) To Edit file in current path,
//    put cursor on first letter of filename (e.g. "a" of "abc.txt")
//
//    abc.txt ...line-data... def.txt ...line-data... ghi.txt
//
// 5) Press mapped key from step 1.
// 6) You are now in Edit on that file name.
//--------------------------------------------------------------------------------------
void EditFileAtCursor(void) {

  char *line_data;
  char *filename;
  char *command;
  int   original_line_number;
  int   line_length;
  int   cur_line_number;
  int   col_number_start;
  int   col_number_end;
  int   filename_len;
  int   find_count;

  line_data = spfservice("query","line_data");              // get line data
  line_length = spfservice("query","line_length");          // get line length
  original_line_number = spfservice("query","line_number"); // get line number
  if (original_line_number > 0) {
    col_number_start = spfservice("query","col_number");    // get cursor col
    if (col_number_start < line_length) {
      spfservice("cmd","FIND C' '");                        // find next blank in current line
      find_count = spfservice("query","FIND_STRING_COUNT");
      cur_line_number = spfservice("query","line_number");  // get line number
      if (find_count > 0 && cur_line_number == original_line_number)
        col_number_end = spfservice("query","col_number");  // get blank col number
      else
        col_number_end = line_length + 1;                   // file name at end of line
      filename_len = col_number_end - col_number_start;     // get length of filename
      filename = strgetsubstr(line_data,col_number_start,filename_len); // extract filename
      spfservice("cmd","RESET");                            // reset FIND highlight
      command = strcompose("EDIT ",filename);               // synthesize EDIT command
      spfservice("cmd",command);                            // edit file
      }
    else {
      msg("EditFileAtCursor: cursor must be within text on data line.");
    }
    }
  else {
    msg("EditFileAtCursor: cursor must be on data line.");
  }

} // EditFileAtCursor
