//------------------------------------------------------------------------
// It is difficult to dream up unique passwords that do not contain
// family names, pet names, streets you lived on, birth years, and other
// common words. The more passwords you have, the more of a pain in the
// neck it is to dream up new ones.
//
// This macro generates a unique password with no meaning or context
// and thus harder to "crack".
//
// Using your own name or email address for your login "User ID"s is also
// begging for trouble. To be extra safe, generate a key for your login
// "User ID"s as well.
//
// If you have a lot of passwords, you probably keep them in a password
// file.  It is advisable that your password file be encrypted.
//
// In addition to encryption, a variety of techniques may be used to
// disguise the meaning of passwords within a password file (i.e. which
// password belongs to which login, etc). We will not discuss those
// techniques here. Our main purpose is to help you generate secure
// passwords.
//
// This macro generates the new password directly into your password file
// (or any other file) at the current cursor position just as if you had
// typed it in.
//
// Macro supports 2 parameters in any order:
//
//    Parameter 1 - Password Type, optional, default = EZ
//
//         EZ         generates a password consisting of a prefix followed by four letters.
//                    All letters will be lowercase from top or middle letter row of the
//                    keyboard.
//
//                    PREFIX will be one of:
//
//                       +  prefix, type 4 character quad right then down (clockwise)
//                       -  prefix, type 4 character quad left then down (counter-clockwise)
//
//                    For example, for password,
//
//                       +hers
//
//                    Type the following 4 character quads,
//
//                       hjmn erfd rtgf sdcx
//
//         STRONG     generates a strong password consisting of
//                    random characters from set:
//
//                       a..z, A..Z, 0..9
//
//    Parameter 2 - Password Length, optional, minimum = 8, maximum = 48, default = 16
//
//       If specified, must be even multiple of 4,
//
// To generate a new password:
//
//    1) Edit your password file.
//
//    2) Type in the primary command field.
//
//       NEWPSWD  [...pswd-type...]  [...pswd-length...]
//
//    3) Position the cursor in the password file where you want the
//       new password generated.
//
//    4) Press "3270-Enter".
//
//------------------------------------------------------------------------
void NewPswd(int argc, char *argv[]) {

  char *P1;
  char *P2;
  char *context;
  char *pswd_type;

  int   pswd_len = 0;
  int   min_pswd_len = 8;  // MIN password length
  int   max_pswd_len = 48; // MAX password length
  int   i;

  bool  any_error  = FALSE;

  if (argv[1] == "?") {
    Msg("NEWPSWD parameters, in any order:\n\nPassword Type\nOptional\nEZ | STRONG\nDefault = EZ\n\nPassword Length\nOptional\nMinimum = 8\nMaximum = 32\nDefault = 16");
    }
  else {
    switch (argc) {
      case 2:  P1 = argv[1];               break; // P1 specified
      case 3:  P1 = argv[1]; P2 = argv[2]; break; // P1 and P2 specified
    } // switch
    if (strlen(P1) > 0) {
      if (strisdigit(P1)) // is P1 a number?
        pswd_len = P1;    // yes, P1 is pswd length
      else
        pswd_type = P1;   // no, P1 is pswd type
    }
    if (strlen(P2) > 0) {
      if (strisdigit(P2)) // is P2 a number?
        pswd_len = P2;    // yes, P2 is pswd length
      else
        pswd_type = P2;   // no, P2 is pswd type
    }
    if (strlen(pswd_type) == 0)
      pswd_type = "EZ";
    if (strlen(pswd_len) == 0)
      pswd_len = 16;
    strupr(pswd_type);
    if (pswd_len == 0)
      pswd_len = 16;
    while ((pswd_len % 4) != 0) { // pswd length divisible by 4?
      ++pswd_len;                 // no, bump 1
    } // while
    if (pswd_len < min_pswd_len) {
      pswd_len = min_pswd_len;
      Msg(strcompose("Minimum password length is ",min_pswd_len));
    }
    if (pswd_len > max_pswd_len) {
      pswd_len = max_pswd_len;
      Msg(strcompose("Maximum password length is ",max_pswd_len));
    }
    context = SPFservice("query","context");
    if (context == "EDIT") {
      i = SPFservice("query","line_number");
      if (i == 0) {
        any_error = TRUE;
        Msg("NEWPSWD:  Cursor must be on a data line when this macro is invoked.");
        }
      else {
        i = SPFservice("query","col_number");
        if (i == 0) {
          any_error = TRUE;
          Msg("NEWPSWD:  Cursor must be on a data column when this macro is invoked.");
        }
      }
      }
    else {
      any_error = TRUE;
      Msg("NEWPSWD:  You must be in Edit on a file to invoke this macro.");
    }
    if (!any_error) {
      if (pswd_type == "EZ") {
        if (pswd_len != 16) {
          pswd_len = 16;
          Msg("EZ password length is always 16.");
        }
        NewPswdEZ(); // length always 16
        }
      else {
        if (pswd_type == "STRONG") {
          NewPswdStrong(pswd_len);
          }
        else {
          Msg("NEWPSWD, password type, must be one of:\n\nEZ\nSTRONG");
        }
      }
    }
  }

} // NewPswd

//----------
// NewPswdEZ
//----------
void NewPswdEZ(void) {

  // 3 byte entries, Byte1 = Char, Byte2 = Clockwise boolean, Byte 3 = Counter-clockwise boolean
  char *char_array[];
  int   char_used[]; // 1 = char used (prevent dups)

  char *new_pswd;
  char *cur_char_entry;
  char *cur_char;
  char *cur_char_flag;
  int   coin_toss;
  int   pswd_len;
  int   max_char_index; // must equal index to last char on row 2
  int   i;
  int   clockwise;
  bool  char_ok;

  i = 0;
  //------------------------
  // top letter keyboard row
  //------------------------
  char_array[i] = "q10"; ++i;
  char_array[i] = "w11"; ++i;
  char_array[i] = "e11"; ++i;
  char_array[i] = "r11"; ++i;
  char_array[i] = "t11"; ++i;
  char_array[i] = "y11"; ++i;
  char_array[i] = "u11"; ++i;
  char_array[i] = "i11"; ++i;
  char_array[i] = "o01"; ++i;
  char_array[i] = "p00"; ++i;

  //---------------------------
  // middle letter keyboard row
  //---------------------------
  char_array[i] = "a10"; ++i;
  char_array[i] = "s11"; ++i;
  char_array[i] = "d11"; ++i;
  char_array[i] = "f11"; ++i;
  char_array[i] = "g11"; ++i;
  char_array[i] = "h11"; ++i;
  char_array[i] = "j01"; ++i;
  char_array[i] = "k00"; ++i;
  char_array[i] = "l00"; ++i;

  max_char_index = (i - 1);              // index to last char in "char_array[]"
  coin_toss = RandInRange(99);           // get random number 0..99
  clockwise = coin_toss % 2;             // will result in either 0 or 1
  if (clockwise == 1)
    strcpy(new_pswd,"+");                // indicate clockwise quads
  else
    strcpy(new_pswd,"-");                // indicate counter-clockwise quads
  pswd_len = 0;
  i = RandInRange(max_char_index);       // get random index to char_array[]
  while (pswd_len < 4) {
    char_ok = FALSE;                     // reset
    if (char_used[i] == 1) {             // char already used?
      cur_char_flag = "0";               // prevent duplicate char
      }
    else {
      cur_char_entry = char_array[i];    // get char entry
      cur_char = strgetsubstr(cur_char_entry,1,1); // get char
      if (clockwise == 1)
        cur_char_flag = strgetsubstr(cur_char_entry,2,1); // get clockwise flag
      else
        cur_char_flag = strgetsubstr(cur_char_entry,3,1); // get counter-clockwise flag
    }
    if (cur_char_flag == "1") {
      char_used[i] = 1;                  // set "char used"
      strcat(new_pswd,cur_char);         // catenate char to password
      ++pswd_len;                        // incr password length
      i = RandInRange(max_char_index);   // get random index to char_array[]
      }
    else {
      ++i;                               // step to next "char in array"
      if (i > max_char_index)            // out of range?
        i = RandInRange(max_char_index); // get random index to "char_array[]"
    }
  } // while
  SPFservice("cmd",strcompose("DATA ",new_pswd));

} // NewPswdEZ

//--------------
// NewPswdStrong
//--------------
void NewPswdStrong(int pswd_len) {

  char *char_set = "abcdefghijklmnopqrstuvwxyz"
                   "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                   "01234567890123456789012345"; // repeat numbers for equal distribution

  char *new_pswd = ""; // init to NULL
  char *final_pswd;
  char *rand_char;
  char *parm;

  int   max_chars_in_set;
  int   random_range;
  int   i;

  max_chars_in_set = strlen(char_set);
  while (strlen(new_pswd) < pswd_len) {
    rand_char = GetRandLetter(char_set);
    if (strfindsubstr(new_pswd,rand_char,1) == 0)  // new rand char unique?
      strcat(new_pswd,rand_char);                  // yes, use it
  } // while
  final_pswd = strgetsubstr(new_pswd,1,pswd_len);  // extract number of PSWD chars requested
  SPFservice("cmd",strcompose("DATA ",final_pswd));

} // NewPswdStrong

//--------------
// GetRandLetter
//--------------
char *GetRandLetter(char *rand_set) {

  char *rand_char;
  int   i;
  int   rand_select;

  i = strlen(rand_set);         // length of letter set
  --i;                          // make base 0
  rand_select = RandInRange(i); // get selector
  ++rand_select;                // make base 1
  rand_char = strgetsubstr(rand_set,rand_select,1); // extract random char
  return(rand_char);

} // GetRandLetter
