//------------------------------------------------------------------------------
// Use this edit macro to launch an edit of the file name under the current
// cursor position. This macro is most often used when editing COBOL or C
// program source files that refer to a COBOL copy book or C include file.
//
// Set up a PF key, CTRL key, or ALT key to invoke this macro. Place the cursor
// anywhere on the text of the referenced file name and press the assigned key
// to edit the referenced file.
//
// If the file name is not fully qualified (path+filename), it is assumed
// to be in the same directory as the current edit file.
//
//------------------------------------------------------------------------------

//------------
// Global Vars
//------------
BOOL g_name_has_ext = FALSE; // file name has extension ("name.ext")

//--------
// EditCpy
//--------
void EditCpy(void) {

  char *cur_line;
  char *cur_char;
  char *filename;
  char *command;

  int   cur_col;
  int   last_col;
  int   name_first_col, name_last_col;
  int   namelen;

  cur_line = SPFservice("query","LINE_DATA");
  cur_col  = SPFservice("query","COL_NUMBER");
  last_col = strlen(cur_line);
  if (cur_col > 0 && last_col > cur_col) {
    //------------------------------
    // find left side name delimiter
    //------------------------------
    cur_char = StrGetSubstr(cur_line,cur_col,1);
    while (cur_col > 0 && IsCharOK(cur_char)) {
      --cur_col;
      cur_char = StrGetSubstr(cur_line,cur_col,1);
    } // while
    if (cur_col > 0) {
      ++cur_col; // step over left delimiter
      name_first_col = cur_col;
      //-------------------------------
      // find right side name delimiter
      //-------------------------------
      cur_char = StrGetSubstr(cur_line,cur_col,1);
      while (cur_col > 0 && IsCharOK(cur_char)) {
        ++cur_col;
        cur_char = StrGetSubstr(cur_line,cur_col,1);
      } // while
      if (cur_col <= last_col) {
        --cur_col; // back up over right delimiter
        name_last_col = cur_col;
        namelen = name_last_col - name_first_col + 1;
        filename = StrGetSubstr(cur_line,name_first_col,namelen);
        if (!g_name_has_ext) {
          //----------------------------------
          // file name does not have extension
          //----------------------------------
          strcat(filename,".cpy"); // catenate default extension
        }
        command = StrCompose("edit ",filename);
        //----------
        // edit file
        //----------
        SPFservice("cmd",command);
        }
      else {
        Msg("Right side delimiter for file name must be single quote, double quote, or >.");
      }
      }
    else {
      Msg("Left side delimiter for file name must be single quote, double quote, or <.");
    }
    }
  else {
    Msg("Put cursor on file name before invoking EDITCPY macro.");
  }

} // EditCpy

//---------
// IsCharOK
//---------
BOOL IsCharOK(char *cur_char) {

  // To be "OK" char must be one of the following:
  //
  //     alphanumeric (a-z, A-Z, 0-9)
  //     underscore   (_)
  //     backslash    (\)
  //     hyphen       (-)
  //     period       (.)

  BOOL char_OK = FALSE;

  if ( StrIsAlnum(cur_char)
    || cur_char == "_"
    || cur_char == "\\"
    || cur_char == "-"
    || cur_char == "." ) {
    char_OK = TRUE;
    if (cur_char == "." ) {
      g_name_has_ext = TRUE;
    }
  }
  return(char_OK);

} // IsCharOK
