//-----------------------------------------------------------------------
// Macro: FINDWORD
//
// Function:
//
//   This macro finds the word at the current cursor position (preloads the
//   FIND string). Now you can press F5(Rfind) to find subsequent instances of
//   that word.
//
// NOTE: Most example macros provide trivial functionality. Their primary
// purpose is to show a variety of coding techniques.
//-----------------------------------------------------------------------
void findword(void) {

  char *text;
  char *word;
  char *command;

  int   move_left_cols = 0;
  int   col;
  int   col_start, col_end;
  int   line_length;
  int   word_length;

  text = SPFservice("query","line_data");          // get current line text
  col = SPFservice("query","col_number");          // get current data column number
  line_length = SPFservice("query","line_length"); // get current line length
  if (text && col >= 1 && col <= line_length) {
    while (col > 1 && StrGetSubstr(text,col,1) != " ") {
      ++move_left_cols;
      --col; // back up to blank
    } // while
    if (StrGetSubstr(text,col,1) == " ") ++col;    // step over blank
    col_start = col;                               // save for later
    col_end = StrFindSubstr(text," ",col_start);   // find next blank
    if (col_end > col_start) {
      //-------------
      // "word" found
      //-------------
      word_length = (col_end - col_start) + 1;     // word length
      }
    else {
      //---------------------------------------------------------------
      // "word" not found, make "word" = preceding blank to end of line
      //---------------------------------------------------------------
      if (col_start > 1) --col_start;              // back up over preceding blank
      col_end = line_length;
      word_length = (col_end - col_start) + 1;     // word length
    }
    //--------------
    // find the word
    //--------------
    command = strcompose("CURSORMOVE LEFT ",move_left_cols);
    SPFservice("cmd",command);
    word = StrGetSubstr(text,col_start,word_length);
    command = strcompose("FIND ",word);
    SPFservice("cmd",command);
    }
  else {
    Msg("FINDWORD: no word at current cursor position.");
  }

} // findword
