//---------------------------------------------------------------------------------
// This is an example of how a user might initialize and launch a simple dialog.
// The sample dialog "SpfDialog.user1.txt" has a variety of field types. The dialog
// definition specifies the macro to get control when the dialog commits.
//
// If the dialog is cancelled, normal SPF/SE processing resumes.
//
// In all cases, the call to SPFservice "dialog" must be the last statement
// in the macro which launches a dialog.
//
// NOTE1: User dialogs are modal. They must be completed before any other SPF
//        operation can be performed. Things you can not do when a user dialog
//        is active:
//
//        * Not allowed to jump to an option menu (e.g. "=0.x")
//        * Not allowed to launch an SPF dialog
//        * Not allowed to launch a File List
//        * Not allowed to Browse/Edit/View a file
//        * Not allowed to access HELP
//        * Not allowed to change SPF window size
//        * If menu style = "MenuBar", not allowed to access MenuBar.
//
//        The only way to complete a user dialog is one of,
//
//        * Press 3270Enter to commit the dialog
//        * Press F3 to end the dialog with no action
//
// NOTE2: The name of this macro does not have to match the dialog name.
//---------------------------------------------------------------------------------
void User1(void) {

  spfservice("vput","User1.TextIn","this is some input text"); // initialize text var
  spfservice("vput","User1.NumIn","99");                       // initialize number var
  spfservice("vput","User1.ChoiceIn","Choice2");               // initialize choice var
  spfservice("vput","User1.BoolIn","TRUE");                    // initialize boolean var
  spfservice("vput","User1.List.0","5");                       // set number of items in list
  // set list items
  spfservice("vput","User1.List.1","aaa (field1 content)   bbb (field2 content)");
  spfservice("vput","User1.List.2","ccc (field1 content)   ddd (field2 content)");
  spfservice("vput","User1.List.3","eee (field1 content)   fff (field2 content)");
  spfservice("vput","User1.List.4","ggg (field1 content)   hhh (field2 content)");
  spfservice("vput","User1.List.5","iii (field1 content)   jjj (field2 content)");
  spfservice("dialog","User1");                                // launch dialog

  // When the dialog commits via 3270ENTER key, macro "User1Commit.c" gets control

} // User1
